package enseirb.myinpulse;

import static enseirb.myinpulse.model.ProjectDecisionValue.*;

import static org.junit.jupiter.api.Assertions.*;

import enseirb.myinpulse.model.Entrepreneur;
import enseirb.myinpulse.model.Project;
import enseirb.myinpulse.model.SectionCell;
import enseirb.myinpulse.service.EntrepreneurApiService;
import enseirb.myinpulse.service.database.EntrepreneurService;
import enseirb.myinpulse.service.database.ProjectService;
import enseirb.myinpulse.service.database.SectionCellService;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.server.ResponseStatusException;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@SpringBootTest
@Transactional
public class EntrepreneurApiServiceTest {
    private static Entrepreneur entrepreneur;
    private static Project project;
    private static Iterable<SectionCell> sectionCells2;
    private static Iterable<SectionCell> sectionCells3;
    @Autowired private EntrepreneurApiService entrepreneurApiService;
    @Autowired private EntrepreneurService entrepreneurService;
    @Autowired private ProjectService projectService;
    @Autowired private SectionCellService sectionCellService;

    @BeforeAll
    static void setup(
            @Autowired EntrepreneurService entrepreneurService,
            @Autowired ProjectService projectService,
            @Autowired SectionCellService sectionCellService) {
        entrepreneur =
                entrepreneurService.addEntrepreneur(
                        new Entrepreneur(
                                "entre",
                                "preneur",
                                "entrepreneur@mail.fr",
                                "entrepreneur2@mail.fr",
                                "01 45 71 25 48",
                                "ENSEIRB",
                                "Info",
                                false));
        entrepreneurService.addEntrepreneur(
                new Entrepreneur(
                        "entre2",
                        "preneur2",
                        "testentrepreneur@mail.fr",
                        "testentrepreneur2@mail.fr",
                        "",
                        "ENSEGID",
                        "",
                        true));
        project =
                projectService.addNewProject(
                        new Project("Project", null, LocalDate.now(), ACTIVE, null, entrepreneur));
        entrepreneurService.updateEntrepreneurProjectProposed(entrepreneur.getIdUser(), project);
        entrepreneurService.updateEntrepreneurProjectParticipation(
                entrepreneur.getIdUser(), project);
        SectionCell s1 =
                sectionCellService.addNewSectionCell(
                        new SectionCell(
                                null,
                                2L,
                                "contenu très intéressant",
                                LocalDateTime.now(),
                                project));

        SectionCell s2 =
                sectionCellService.addNewSectionCell(
                        new SectionCell(
                                null,
                                3L,
                                "contenu très intéressant2",
                                LocalDateTime.now(),
                                project));
        sectionCells2 = sectionCellService.getSectionCellsByProject(project, 2L);
        sectionCells3 = sectionCellService.getSectionCellsByProject(project, 3L);
    }

    private <T> List<T> IterableToList(Iterable<T> iterable) {
        List<T> l = new ArrayList<>();
        iterable.forEach(l::add);
        return l;
    }

    @Test
    void editValidSectionCell() {
        entrepreneurApiService.editSectionCell(
                IterableToList(sectionCells2).getLast().getIdSectionCell(),
                "modified content",
                "entrepreneur@mail.fr");
        // We get the data from the database again.
        SectionCell s =
                IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).getLast();
        assertEquals("modified content", s.getContentSectionCell());
        assertEquals(
                2, IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).size());
    }

    @Test
    void editInvalidSectionCell() {
        assertThrows(
                ResponseStatusException.class,
                () ->
                        entrepreneurApiService.editSectionCell(
                                -1L, "should not be modified", "entrepreneur@mail.fr"));
        SectionCell s =
                IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).getLast();
        assertEquals("contenu très intéressant", s.getContentSectionCell());
        assertEquals(
                1, IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).size());
    }

    @Test
    void editSectionCellInvalidAccess() {
        assertThrows(
                ResponseStatusException.class,
                () ->
                        entrepreneurApiService.editSectionCell(
                                IterableToList(sectionCells3).getFirst().getIdSectionCell(),
                                "should not be modified",
                                "testentrepreneur@mail.fr"));
        SectionCell s =
                IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).getFirst();

        assertEquals("contenu très intéressant", s.getContentSectionCell());
    }

    @Test
    void removeValidSectionCell() {
        SectionCell tmpCell =
                sectionCellService.addNewSectionCell(
                        new SectionCell(
                                null, 2L, "contenu temporaire", LocalDateTime.now(), project));
        assertEquals(
                2, IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).size());
        assertDoesNotThrow(
                () ->
                        entrepreneurApiService.removeSectionCell(
                                tmpCell.getIdSectionCell(), "entrepreneur@mail.fr"));
        assertEquals(
                tmpCell.getIdReference(),
                IterableToList(sectionCellService.getSectionCellsByProject(project, -1L))
                        .getLast()
                        .getIdReference());
    }

    @Test
    void removeInvalidSectionCell() {
        assertThrows(
                ResponseStatusException.class,
                () -> entrepreneurApiService.removeSectionCell(-1L, "entrepreneur@mail.fr"));
        SectionCell s =
                IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).getFirst();

        assertEquals("contenu très intéressant", s.getContentSectionCell());
    }

    @Test
    void addValidSectionCell() {
        SectionCell added =
                sectionCellService.addNewSectionCell(
                        new SectionCell(null, 2L, "contenu ajouté", LocalDateTime.now(), project));
        entrepreneurApiService.addSectionCell(added, "entrepreneur@mail.fr");
        SectionCell s =
                IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).getLast();
        assertEquals("contenu ajouté", s.getContentSectionCell());
        assertEquals(
                2, IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).size());
    }

    @Test
    void addSectionCellInvalidAccess() {
        SectionCell added =
                new SectionCell(null, 2L, "contenu ajouté", LocalDateTime.now(), project);
        assertThrows(
                ResponseStatusException.class,
                () -> entrepreneurApiService.addSectionCell(added, "fauxentrepreneur@mail.fr"));
        SectionCell s =
                IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).getLast();
        assertEquals(
                1, IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).size());
        assertEquals("contenu très intéressant", s.getContentSectionCell());
    }

    @Test
    void addInvalidSectionCell() {
        SectionCell added =
                sectionCellService.addNewSectionCell(
                        new SectionCell(null, -1L, "contenu ajouté", LocalDateTime.now(), project));
        assertThrows(
                ResponseStatusException.class,
                () -> entrepreneurApiService.addSectionCell(added, "entrepreneur@mail.fr"));
    }

    @Test
    void requestValidProject() {}
}
